import { reactive, computed, watch } from 'vue'
import { omit } from 'lodash-es'
import useForm from '@/hooks/useForm'
import useOptions from '@/hooks/useOptions'
import useVW from '@/hooks/useVW'
import { getCurDayRange } from '@/utils/common'
import useFetch from '@/hooks/useFetch'

import type { EpFormItem, EpFormRows } from 'global-type'
import type { ExtractApiParams } from '@/api/api'

type FormMode = Omit<
  ExtractApiParams<'getStatisticsByGroup'>,
  'endTime' | 'startTime' | 'hasGroupLeaderScore' | 'markingGroupNumbers'
> & {
  time: string[]
  hasGroupLeaderScore: string[]
  expand: string[]
  markingGroupNumbers?: number | string
}

const useStatisticsFilter = () => {
  const {
    subjectList,
    mainQuestionList,
    groupListWithAll,
    changeModelValue,
    dataModel,
    onOptionInit,
    isExpert,
    isLeader,
  } = useOptions(['subject', 'question', 'group'])
  const curDayRange = getCurDayRange()
  const model = reactive<any>({
    subjectCode: dataModel.subject || '',
    questionMainNumber: dataModel.question,
    markingGroupNumbers: dataModel.group,
    time: [curDayRange[0], curDayRange[1]],
    // hasGroupLeaderScore: ['true'],
    hasGroupLeaderScore: [],
    expand: [],
    markerId: '',
    online: '',
  })
  const { fetch: getMarkerList, result: markerList } = useFetch('getMarkerList')
  watch(
    () => [model.subjectCode, model.questionMainNumber, model.markingGroupNumbers],
    () => {
      if (model.subjectCode && model.questionMainNumber) {
        getMarkerList({
          subjectCode: model.subjectCode,
          mainNumber: model.questionMainNumber,
          markingGroupNumber: model.markingGroupNumbers,
        })
        model.markerId = ''
      }
    },
    {
      immediate: true,
    }
  )
  // const fetchModel = computed<ExtractApiParams<'getStatisticsByGroup'> & { expand: boolean }>(() => {
  const fetchModel = computed<any>(() => {
    return Object.assign(omit(model, 'time'), {
      startTime: model.time?.[0],
      endTime: model.time?.[1],
      hasGroupLeaderScore: model.hasGroupLeaderScore.length > 0,
      expand: model.expand.length > 0,
      markingGroupNumbers:
        typeof model.markingGroupNumbers === 'number' ? [model.markingGroupNumbers] : model.markingGroupNumbers,
    })
  })

  const { formRef, elFormRef, defineColumn, _ } = useForm()

  watch(dataModel, () => {
    model.subjectCode = dataModel.subject || ''
    model.questionMainNumber = dataModel.question
    model.markingGroupNumbers = dataModel.group
  })

  const rows: EpFormRows = reactive({
    'row-1': {},
    'row-2': {},
  })

  const OneRowSpan5 = defineColumn(_, 'row-1', { span: 5 })
  const OneRowSpan4 = defineColumn(_, 'row-1', { span: 4 })
  const OneRowSpan3 = defineColumn(_, 'row-1', { span: 3 })
  const OneRowSpan1 = defineColumn(_, 'row-1', { span: 1 })
  const TwoRowSpan5 = defineColumn(_, 'row-2', { span: 5 })
  const TwoRowSpan8 = defineColumn(_, 'row-2', { span: 8 })
  const TwoRowSpan3 = defineColumn(_, 'row-2', { span: 3 })
  const TwoRowSpan4 = defineColumn(_, 'row-2', { span: 4 })

  const items = computed<EpFormItem[]>(() => [
    OneRowSpan4({
      labelWidth: '52px',
      label: '科目',
      prop: 'subjectCode',
      slotType: 'select',
      slot: {
        options: subjectList.value,
        onChange: changeModelValue('subject'),
        disabled: !isExpert.value,
      },
    }),
    OneRowSpan4({
      labelWidth: '52px',
      prop: 'questionMainNumber',
      label: '大题',
      slotType: 'select',
      slot: {
        options: mainQuestionList.value,
        onChange: changeModelValue('question'),
        disabled: !isExpert.value && !isLeader.value,
      },
    }),
    OneRowSpan4({
      labelWidth: '52px',
      prop: 'markingGroupNumbers',
      label: '小组',
      slotType: 'select',
      slot: {
        options: groupListWithAll.value,
        onChange: changeModelValue('group'),
      },
    }),
    OneRowSpan4({
      labelWidth: '65px',
      label: '评卷员',
      slotType: 'select',
      prop: 'markerId',
      slot: {
        options: [
          { label: '全部', value: '' },
          ...(markerList?.value || []).map((marker) => ({
            value: marker.id,
            label: marker.loginName ? `${marker.loginName}-${marker.name}` : marker.name,
          })),
        ],
        filterable: true,
        clearable: true,
      },
    }),
    OneRowSpan4({
      labelWidth: '20px',
      slotName: 'button-group',
    }),

    OneRowSpan4({
      labelWidth: '0px',
      slotName: 'columns-setting',
    }),
    TwoRowSpan8({
      labelWidth: '52px',
      label: '时间',
      slotType: 'dateTime',
      prop: 'time',
      slot: {
        type: 'datetimerange',
        valueFormat: 'YYYYMMDDHHmmss',
        startPlaceholder: '开始时间',
        endPlaceholder: '结束时间',
      },
    }),
    TwoRowSpan4({
      labelWidth: '52px',
      prop: 'online',
      label: '人员',
      slotType: 'select',
      slot: {
        options: [
          { label: '全部', value: '' },
          { label: '在线', value: true },
          { label: '离线', value: false },
        ],
      },
    }),
    TwoRowSpan3({
      prop: 'expand',
      label: '展开明细',
      slotType: 'checkbox',
      slot: {
        options: [{ label: '按人员展开' }],
      },
    }),
    TwoRowSpan3({
      prop: 'hasGroupLeaderScore',
      label: '组长给分',
      slotType: 'checkbox',
      slot: {
        options: [{ label: 'true', slotLabel: '包含' }],
      },
    }),
  ])

  return {
    formRef,
    elFormRef,
    model,
    fetchModel,
    items,
    rows,
    onOptionInit,
  }
}

export default useStatisticsFilter
